from typing import Any, Dict, List, Literal, Optional, Union, Generic, TypeVar
from pydantic import BaseModel
from .tool import Tool, ApiTool  # noqa: F401

OutputT = TypeVar("OutputT")


# Message part types
class TextPart(BaseModel):
    type: Literal["text"] = "text"
    text: str


class ImagePart(BaseModel):
    type: Literal["image"] = "image"
    image: str  # Base64 encoded image or URL
    mime_type: Optional[str] = None


class ToolCallPart(BaseModel):
    type: Literal["tool-call"] = "tool-call"
    id: Optional[str] = None
    tool_call_id: str
    tool_name: str
    safety_checks: Optional[List[Any]] = None
    args: Dict[str, Any]


class ToolResultPart(BaseModel):
    type: Literal["tool-result"] = "tool-result"
    tool_call_id: str
    tool_name: str
    result: Any
    is_error: Optional[bool] = False

class ReasoningPart(BaseModel):
    type: Literal["reasoning"] = "reasoning"
    id: Optional[str] = None
    reasoning: str
    signature: Optional[str] = None
    instructions: Optional[str] = None

class UserMessage(BaseModel):
    role: Literal["user"] = "user"
    content: List[Union[TextPart, ImagePart]]


class AssistantMessage(BaseModel):
    role: Literal["assistant"] = "assistant"
    content: List[Union[TextPart, ToolCallPart, ReasoningPart]]
    response_id: Optional[str] = None


class ToolMessage(BaseModel):
    role: Literal["tool"] = "tool"
    content: List[ToolResultPart]


Message = Union[UserMessage, AssistantMessage, ToolMessage]


# Request/Response models
class Model(BaseModel):
    provider: Literal["anthropic", "openai", "herd"]
    name: str
    api_key: Optional[str] = None


class SingleActRequest(BaseModel):
    model: Model
    system: Optional[str] = None
    messages: Optional[List[Message]] = None
    tools: Optional[List[ApiTool]] = None
    temperature: Optional[float] = None
    max_tokens: Optional[int] = None


class TokenUsage(BaseModel):
    prompt_tokens: int
    completion_tokens: int
    total_tokens: int


class SingleActResponse(BaseModel):
    message: AssistantMessage
    finish_reason: Literal[
        "stop", "length", "content-filter", "tool-calls", "error", "other", "unknown"
    ]
    usage: Optional[TokenUsage] = None


# Step definition
class Step(BaseModel):
    text: str
    response_id: Optional[str] = None
    reasoning_parts: Optional[List[ReasoningPart]] = None
    tool_calls: Optional[List[ToolCallPart]] = None
    tool_results: Optional[List[ToolResultPart]] = None
    finish_reason: Optional[
        Literal[
            "stop",
            "length",
            "content-filter",
            "tool-calls",
            "error",
            "other",
            "unknown",
        ]
    ] = None
    usage: Optional[TokenUsage] = None


# Act response
class ActResponse(BaseModel, Generic[OutputT]):
    messages: List[Message]
    steps: List[Step]
    text: Optional[str] = None
    output: OutputT
    usage: Optional[TokenUsage] = None



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Command,
    Request,
    Request_ClickMouse,
    Request_DragMouse,
    Request_GetCursorPosition,
    Request_MoveMouse,
    Request_PressKey,
    Request_Scroll,
    Request_TakeScreenshot,
    Request_TypeText,
    Request_Wait,
)

__all__ = [
    "Command",
    "Request",
    "Request_ClickMouse",
    "Request_DragMouse",
    "Request_GetCursorPosition",
    "Request_MoveMouse",
    "Request_PressKey",
    "Request_Scroll",
    "Request_TakeScreenshot",
    "Request_TypeText",
    "Request_Wait",
]



from ..types.act import Model
from typing import Literal
from pydantic import Field


class Herd(Model):
    """Model adapter for Herd (Scrapybara-hosted LLMs).

    Args:
        name: Herd model name

    Returns:
        A Model configuration object
    """

    provider: Literal["herd"] = Field(default="herd", frozen=True)

    def __init__(self, name: str) -> None:
        super().__init__(provider="herd", name=name)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.env_get_response import EnvGetResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.env_response import EnvResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class EnvClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EnvGetResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvGetResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.env.get(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/env",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EnvGetResponse,
                    parse_obj_as(
                        type_=EnvGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def set(
        self,
        instance_id: str,
        *,
        variables: typing.Dict[str, str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EnvResponse:
        """
        Parameters
        ----------
        instance_id : str

        variables : typing.Dict[str, str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.env.set(
            instance_id="instance_id",
            variables={"key": "value"},
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/env",
            method="POST",
            json={
                "variables": variables,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EnvResponse,
                    parse_obj_as(
                        type_=EnvResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, instance_id: str, *, keys: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> EnvResponse:
        """
        Parameters
        ----------
        instance_id : str

        keys : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.env.delete(
            instance_id="instance_id",
            keys=["keys"],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/env/delete",
            method="POST",
            json={
                "keys": keys,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EnvResponse,
                    parse_obj_as(
                        type_=EnvResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncEnvClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> EnvGetResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvGetResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.env.get(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/env",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EnvGetResponse,
                    parse_obj_as(
                        type_=EnvGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def set(
        self,
        instance_id: str,
        *,
        variables: typing.Dict[str, str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EnvResponse:
        """
        Parameters
        ----------
        instance_id : str

        variables : typing.Dict[str, str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.env.set(
                instance_id="instance_id",
                variables={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/env",
            method="POST",
            json={
                "variables": variables,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EnvResponse,
                    parse_obj_as(
                        type_=EnvResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, instance_id: str, *, keys: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> EnvResponse:
        """
        Parameters
        ----------
        instance_id : str

        keys : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EnvResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.env.delete(
                instance_id="instance_id",
                keys=["keys"],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/env/delete",
            method="POST",
            json={
                "keys": keys,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EnvResponse,
                    parse_obj_as(
                        type_=EnvResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import ScrapybaraEnvironment
import os
import httpx
from .core.api_error import ApiError
from .core.client_wrapper import SyncClientWrapper
from .instance.client import InstanceClient
from .browser.client import BrowserClient
from .code.client import CodeClient
from .notebook.client import NotebookClient
from .env.client import EnvClient
from .types.deployment_config_instance_type import DeploymentConfigInstanceType
from .core.request_options import RequestOptions
from .types.get_instance_response import GetInstanceResponse
from .core.pydantic_utilities import parse_obj_as
from .errors.unprocessable_entity_error import UnprocessableEntityError
from .types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from .core.jsonable_encoder import jsonable_encoder
from .types.auth_state_response import AuthStateResponse
from .core.client_wrapper import AsyncClientWrapper
from .instance.client import AsyncInstanceClient
from .browser.client import AsyncBrowserClient
from .code.client import AsyncCodeClient
from .notebook.client import AsyncNotebookClient
from .env.client import AsyncEnvClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BaseClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : ScrapybaraEnvironment
        The environment to use for requests from the client. from .environment import ScrapybaraEnvironment



        Defaults to ScrapybaraEnvironment.PRODUCTION



    api_key : typing.Optional[str]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 600 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from scrapybara import Scrapybara

    client = Scrapybara(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: ScrapybaraEnvironment = ScrapybaraEnvironment.PRODUCTION,
        api_key: typing.Optional[str] = os.getenv("SCRAPYBARA_API_KEY"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 600 if httpx_client is None else None
        if api_key is None:
            raise ApiError(
                body="The client must be instantiated be either passing in api_key or setting SCRAPYBARA_API_KEY"
            )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.instance = InstanceClient(client_wrapper=self._client_wrapper)
        self.browser = BrowserClient(client_wrapper=self._client_wrapper)
        self.code = CodeClient(client_wrapper=self._client_wrapper)
        self.notebook = NotebookClient(client_wrapper=self._client_wrapper)
        self.env = EnvClient(client_wrapper=self._client_wrapper)

    def start(
        self,
        *,
        instance_type: typing.Optional[DeploymentConfigInstanceType] = OMIT,
        timeout_hours: typing.Optional[float] = OMIT,
        blocked_domains: typing.Optional[typing.Sequence[str]] = OMIT,
        resolution: typing.Optional[typing.Sequence[int]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInstanceResponse:
        """
        Parameters
        ----------
        instance_type : typing.Optional[DeploymentConfigInstanceType]

        timeout_hours : typing.Optional[float]

        blocked_domains : typing.Optional[typing.Sequence[str]]

        resolution : typing.Optional[typing.Sequence[int]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInstanceResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.start()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/start",
            method="POST",
            json={
                "instance_type": instance_type,
                "timeout_hours": timeout_hours,
                "blocked_domains": blocked_domains,
                "resolution": resolution,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInstanceResponse,
                    parse_obj_as(
                        type_=GetInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInstanceResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.get(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInstanceResponse,
                    parse_obj_as(
                        type_=GetInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_instances(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[GetInstanceResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GetInstanceResponse]
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.get_instances()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/instances",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[GetInstanceResponse],
                    parse_obj_as(
                        type_=typing.List[GetInstanceResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_auth_states(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[AuthStateResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AuthStateResponse]
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.get_auth_states()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/auth_states",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[AuthStateResponse],
                    parse_obj_as(
                        type_=typing.List[AuthStateResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBaseClient:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : ScrapybaraEnvironment
        The environment to use for requests from the client. from .environment import ScrapybaraEnvironment



        Defaults to ScrapybaraEnvironment.PRODUCTION



    api_key : typing.Optional[str]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 600 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from scrapybara import AsyncScrapybara

    client = AsyncScrapybara(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: ScrapybaraEnvironment = ScrapybaraEnvironment.PRODUCTION,
        api_key: typing.Optional[str] = os.getenv("SCRAPYBARA_API_KEY"),
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 600 if httpx_client is None else None
        if api_key is None:
            raise ApiError(
                body="The client must be instantiated be either passing in api_key or setting SCRAPYBARA_API_KEY"
            )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.instance = AsyncInstanceClient(client_wrapper=self._client_wrapper)
        self.browser = AsyncBrowserClient(client_wrapper=self._client_wrapper)
        self.code = AsyncCodeClient(client_wrapper=self._client_wrapper)
        self.notebook = AsyncNotebookClient(client_wrapper=self._client_wrapper)
        self.env = AsyncEnvClient(client_wrapper=self._client_wrapper)

    async def start(
        self,
        *,
        instance_type: typing.Optional[DeploymentConfigInstanceType] = OMIT,
        timeout_hours: typing.Optional[float] = OMIT,
        blocked_domains: typing.Optional[typing.Sequence[str]] = OMIT,
        resolution: typing.Optional[typing.Sequence[int]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInstanceResponse:
        """
        Parameters
        ----------
        instance_type : typing.Optional[DeploymentConfigInstanceType]

        timeout_hours : typing.Optional[float]

        blocked_domains : typing.Optional[typing.Sequence[str]]

        resolution : typing.Optional[typing.Sequence[int]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInstanceResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.start()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/start",
            method="POST",
            json={
                "instance_type": instance_type,
                "timeout_hours": timeout_hours,
                "blocked_domains": blocked_domains,
                "resolution": resolution,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInstanceResponse,
                    parse_obj_as(
                        type_=GetInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInstanceResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.get(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInstanceResponse,
                    parse_obj_as(
                        type_=GetInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_instances(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[GetInstanceResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GetInstanceResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.get_instances()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/instances",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[GetInstanceResponse],
                    parse_obj_as(
                        type_=typing.List[GetInstanceResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_auth_states(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[AuthStateResponse]:
        """
        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AuthStateResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.get_auth_states()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/auth_states",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[AuthStateResponse],
                    parse_obj_as(
                        type_=typing.List[AuthStateResponse],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: ScrapybaraEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AuthStateResponse,
    BashResponse,
    BrowserAuthenticateResponse,
    BrowserGetCdpUrlResponse,
    BrowserGetCurrentUrlResponse,
    Button,
    CellType,
    ClickMouseAction,
    ClickMouseActionClickType,
    ComputerResponse,
    DeploymentConfigInstanceType,
    DragMouseAction,
    EditResponse,
    EnvGetResponse,
    EnvResponse,
    ExecuteCellRequest,
    FileResponse,
    GetCursorPositionAction,
    GetInstanceResponse,
    GetInstanceResponseInstanceType,
    HttpValidationError,
    InstanceGetStreamUrlResponse,
    InstanceScreenshotResponse,
    KernelInfo,
    ModifyBrowserAuthResponse,
    MoveMouseAction,
    Notebook,
    NotebookCell,
    PressKeyAction,
    SaveBrowserAuthResponse,
    ScrollAction,
    StartBrowserResponse,
    Status,
    StopBrowserResponse,
    StopInstanceResponse,
    TakeScreenshotAction,
    TypeTextAction,
    ValidationError,
    ValidationErrorLocItem,
    WaitAction,
)
from .errors import UnprocessableEntityError
from . import browser, code, env, instance, notebook
from .client import AsyncScrapybara, Scrapybara
from .environment import ScrapybaraEnvironment
from .instance import (
    Command,
    Request,
    Request_ClickMouse,
    Request_DragMouse,
    Request_GetCursorPosition,
    Request_MoveMouse,
    Request_PressKey,
    Request_Scroll,
    Request_TakeScreenshot,
    Request_TypeText,
    Request_Wait,
)
from .version import __version__

__all__ = [
    "AsyncScrapybara",
    "AuthStateResponse",
    "BashResponse",
    "BrowserAuthenticateResponse",
    "BrowserGetCdpUrlResponse",
    "BrowserGetCurrentUrlResponse",
    "Button",
    "CellType",
    "ClickMouseAction",
    "ClickMouseActionClickType",
    "Command",
    "ComputerResponse",
    "DeploymentConfigInstanceType",
    "DragMouseAction",
    "EditResponse",
    "EnvGetResponse",
    "EnvResponse",
    "ExecuteCellRequest",
    "FileResponse",
    "GetCursorPositionAction",
    "GetInstanceResponse",
    "GetInstanceResponseInstanceType",
    "HttpValidationError",
    "InstanceGetStreamUrlResponse",
    "InstanceScreenshotResponse",
    "KernelInfo",
    "ModifyBrowserAuthResponse",
    "MoveMouseAction",
    "Notebook",
    "NotebookCell",
    "PressKeyAction",
    "Request",
    "Request_ClickMouse",
    "Request_DragMouse",
    "Request_GetCursorPosition",
    "Request_MoveMouse",
    "Request_PressKey",
    "Request_Scroll",
    "Request_TakeScreenshot",
    "Request_TypeText",
    "Request_Wait",
    "SaveBrowserAuthResponse",
    "Scrapybara",
    "ScrapybaraEnvironment",
    "ScrollAction",
    "StartBrowserResponse",
    "Status",
    "StopBrowserResponse",
    "StopInstanceResponse",
    "TakeScreenshotAction",
    "TypeTextAction",
    "UnprocessableEntityError",
    "ValidationError",
    "ValidationErrorLocItem",
    "WaitAction",
    "__version__",
    "browser",
    "code",
    "env",
    "instance",
    "notebook",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from ..types.start_browser_response import StartBrowserResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.browser_get_cdp_url_response import BrowserGetCdpUrlResponse
from ..types.browser_get_current_url_response import BrowserGetCurrentUrlResponse
from ..types.save_browser_auth_response import SaveBrowserAuthResponse
from ..types.modify_browser_auth_response import ModifyBrowserAuthResponse
from ..types.browser_authenticate_response import BrowserAuthenticateResponse
from ..types.stop_browser_response import StopBrowserResponse
from ..core.client_wrapper import AsyncClientWrapper


class BrowserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def start(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StartBrowserResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartBrowserResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.start(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/start",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StartBrowserResponse,
                    parse_obj_as(
                        type_=StartBrowserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_cdp_url(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BrowserGetCdpUrlResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BrowserGetCdpUrlResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.get_cdp_url(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/cdp_url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BrowserGetCdpUrlResponse,
                    parse_obj_as(
                        type_=BrowserGetCdpUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_current_url(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BrowserGetCurrentUrlResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BrowserGetCurrentUrlResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.get_current_url(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/current_url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BrowserGetCurrentUrlResponse,
                    parse_obj_as(
                        type_=BrowserGetCurrentUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def save_auth(
        self,
        instance_id: str,
        *,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SaveBrowserAuthResponse:
        """
        Parameters
        ----------
        instance_id : str

        name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SaveBrowserAuthResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.save_auth(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/save_auth",
            method="POST",
            params={
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SaveBrowserAuthResponse,
                    parse_obj_as(
                        type_=SaveBrowserAuthResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def modify_auth(
        self,
        instance_id: str,
        *,
        auth_state_id: str,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ModifyBrowserAuthResponse:
        """
        Parameters
        ----------
        instance_id : str

        auth_state_id : str

        name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModifyBrowserAuthResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.modify_auth(
            instance_id="instance_id",
            auth_state_id="auth_state_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/modify_auth",
            method="POST",
            params={
                "auth_state_id": auth_state_id,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModifyBrowserAuthResponse,
                    parse_obj_as(
                        type_=ModifyBrowserAuthResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def authenticate(
        self, instance_id: str, *, auth_state_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> BrowserAuthenticateResponse:
        """
        Parameters
        ----------
        instance_id : str

        auth_state_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BrowserAuthenticateResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.authenticate(
            instance_id="instance_id",
            auth_state_id="auth_state_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/authenticate",
            method="POST",
            params={
                "auth_state_id": auth_state_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BrowserAuthenticateResponse,
                    parse_obj_as(
                        type_=BrowserAuthenticateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def stop(self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> StopBrowserResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StopBrowserResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.browser.stop(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/stop",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StopBrowserResponse,
                    parse_obj_as(
                        type_=StopBrowserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncBrowserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def start(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StartBrowserResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StartBrowserResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.start(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/start",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StartBrowserResponse,
                    parse_obj_as(
                        type_=StartBrowserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_cdp_url(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BrowserGetCdpUrlResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BrowserGetCdpUrlResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.get_cdp_url(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/cdp_url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BrowserGetCdpUrlResponse,
                    parse_obj_as(
                        type_=BrowserGetCdpUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_current_url(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BrowserGetCurrentUrlResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BrowserGetCurrentUrlResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.get_current_url(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/current_url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BrowserGetCurrentUrlResponse,
                    parse_obj_as(
                        type_=BrowserGetCurrentUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def save_auth(
        self,
        instance_id: str,
        *,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SaveBrowserAuthResponse:
        """
        Parameters
        ----------
        instance_id : str

        name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SaveBrowserAuthResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.save_auth(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/save_auth",
            method="POST",
            params={
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    SaveBrowserAuthResponse,
                    parse_obj_as(
                        type_=SaveBrowserAuthResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def modify_auth(
        self,
        instance_id: str,
        *,
        auth_state_id: str,
        name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ModifyBrowserAuthResponse:
        """
        Parameters
        ----------
        instance_id : str

        auth_state_id : str

        name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModifyBrowserAuthResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.modify_auth(
                instance_id="instance_id",
                auth_state_id="auth_state_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/modify_auth",
            method="POST",
            params={
                "auth_state_id": auth_state_id,
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ModifyBrowserAuthResponse,
                    parse_obj_as(
                        type_=ModifyBrowserAuthResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def authenticate(
        self, instance_id: str, *, auth_state_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> BrowserAuthenticateResponse:
        """
        Parameters
        ----------
        instance_id : str

        auth_state_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BrowserAuthenticateResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.authenticate(
                instance_id="instance_id",
                auth_state_id="auth_state_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/authenticate",
            method="POST",
            params={
                "auth_state_id": auth_state_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BrowserAuthenticateResponse,
                    parse_obj_as(
                        type_=BrowserAuthenticateResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def stop(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopBrowserResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StopBrowserResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.browser.stop(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/browser/stop",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StopBrowserResponse,
                    parse_obj_as(
                        type_=StopBrowserResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



from datetime import datetime

UBUNTU_SYSTEM_PROMPT = f"""<SYSTEM_CAPABILITY>
* You have access to an Ubuntu VM with internet connectivity
* You can install Ubuntu applications using the bash tool (use curl over wget)
* To run GUI applications with the bash tool, use a subshell, e.g. "(DISPLAY=:1 xterm &)", make sure to include the parantheses
* GUI apps will appear but may take time to load - confirm with an extra screenshot
* Chromium is the default browser
* Start Chromium via the bash tool "(DISPLAY=:1 chromium &)", but interact with it visually via the computer tool
* If you need to read a HTML file:
  - Open with the address bar in Chromium
* For commands with large text output:
  - Redirect to a temp file
  - Use str_replace_editor or grep with context (-B and -A flags) to view output
* When viewing pages:
  - Zoom out to see full content, or
  - Scroll to ensure you see everything
* When interacting with a field, always clear the field first using "ctrl+A" and "delete"
  - Take an extra screenshot after clicking "enter" to confirm the field is properly submitted and move the mouse to the next field
* Computer function calls take time, string together calls when possible
* You are allowed to take actions on behalf of the user on sites that are authenticated
* If the user asks you to access a site, assume that the user has already authenticated
* To login additional sites, ask the user to use Auth Contexts or the Interactive Desktop
* If first screenshot shows black screen:
  - Click mouse in screen center
  - Take another screenshot
* Today's date is {datetime.today().strftime('%A, %B %d, %Y')}
</SYSTEM_CAPABILITY>

<IMPORTANT>
* If given a complex task, break down into smaller steps and ask the user for details only if necessary
* Read through web pages thoroughly by scrolling down till you have gathered enough info
* Be concise!
</IMPORTANT>"""
"""DEPRECATED — Please import prompts from their respective models instead: `from scrapybara.anthropic import UBUNTU_SYSTEM_PROMPT`"""


BROWSER_SYSTEM_PROMPT = f"""<SYSTEM_CAPABILITY>
* You have access to a Chromium VM with internet connectivity
* Chromium should already be open and running
* You can interact with web pages using the computer tool
* When viewing pages:
  - Zoom out to see full content, or
  - Scroll to ensure you see everything
* When interacting with a field, always clear the field first using "ctrl+A" and "delete"
  - Take an extra screenshot after clicking "enter" to confirm the field is properly submitted and move the mouse to the next field
* Computer function calls take time, string together calls when possible
* You are allowed to take actions on behalf of the user on sites that are authenticated
* If the user asks you to access a site, assume that the user has already authenticated
* To login additional sites, ask the user to use Auth Contexts
* If first screenshot shows black screen:
  - Click mouse in screen center
  - Take another screenshot
* Today's date is {datetime.today().strftime('%A, %B %d, %Y')}
</SYSTEM_CAPABILITY>

<IMPORTANT>
* If given a complex task, break down into smaller steps and ask the user for details only if necessary
* Read through web pages thoroughly by scrolling down till you have gathered enough info
* Be concise!
</IMPORTANT>"""
"""DEPRECATED — Please import prompts from their respective models instead: `from scrapybara.anthropic import BROWSER_SYSTEM_PROMPT`"""


WINDOWS_SYSTEM_PROMPT = f"""<SYSTEM_CAPABILITY>
* You have access to a Windows VM with internet connectivity
* You can interact with the Windows desktop using the computer tool
* GUI apps will appear but may take time to load - confirm with an extra screenshot
* Edge is the default browser
* When viewing pages:
  - Zoom out to see full content, or
  - Scroll to ensure you see everything
* When interacting with a field, always clear the field first using "ctrl+A" and "delete"
  - Take an extra screenshot after clicking "enter" to confirm the field is properly submitted and move the mouse to the next field
* Computer function calls take time, string together calls when possible
* You are allowed to take actions on behalf of the user on sites that are authenticated
* If the user asks you to access a site, assume that the user has already authenticated
* To login additional sites, ask the user to use Auth Contexts or the Interactive Desktop
* If first screenshot shows black screen:
  - Click mouse in screen center
  - Take another screenshot
* Today's date is {datetime.today().strftime('%A, %B %d, %Y')}
</SYSTEM_CAPABILITY>

<IMPORTANT>
* If given a complex task, break down into smaller steps and ask the user for details only if necessary
* Read through web pages thoroughly by scrolling down till you have gathered enough info
* Be concise!
</IMPORTANT>"""
"""DEPRECATED — Please import prompts from their respective models instead: `from scrapybara.anthropic import WINDOWS_SYSTEM_PROMPT`"""



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.instance_screenshot_response import InstanceScreenshotResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.instance_get_stream_url_response import InstanceGetStreamUrlResponse
from .types.request import Request
from ..types.computer_response import ComputerResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.bash_response import BashResponse
from .types.command import Command
from ..types.edit_response import EditResponse
from ..types.file_response import FileResponse
from ..types.stop_instance_response import StopInstanceResponse
from ..types.get_instance_response import GetInstanceResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InstanceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def screenshot(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InstanceScreenshotResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InstanceScreenshotResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.screenshot(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/screenshot",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    InstanceScreenshotResponse,
                    parse_obj_as(
                        type_=InstanceScreenshotResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_stream_url(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InstanceGetStreamUrlResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InstanceGetStreamUrlResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.get_stream_url(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/stream_url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    InstanceGetStreamUrlResponse,
                    parse_obj_as(
                        type_=InstanceGetStreamUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def computer(
        self, instance_id: str, *, request: Request, request_options: typing.Optional[RequestOptions] = None
    ) -> ComputerResponse:
        """
        Parameters
        ----------
        instance_id : str

        request : Request

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComputerResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara
        from scrapybara.instance import Request_MoveMouse

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.computer(
            instance_id="instance_id",
            request=Request_MoveMouse(
                coordinates=[1],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/computer",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=Request, direction="write"),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ComputerResponse,
                    parse_obj_as(
                        type_=ComputerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def bash(
        self,
        instance_id: str,
        *,
        command: typing.Optional[str] = OMIT,
        restart: typing.Optional[bool] = OMIT,
        get_background_processes: typing.Optional[bool] = OMIT,
        kill_pid: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BashResponse:
        """
        Parameters
        ----------
        instance_id : str

        command : typing.Optional[str]

        restart : typing.Optional[bool]

        get_background_processes : typing.Optional[bool]

        kill_pid : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BashResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.bash(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/bash",
            method="POST",
            json={
                "command": command,
                "restart": restart,
                "get_background_processes": get_background_processes,
                "kill_pid": kill_pid,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BashResponse,
                    parse_obj_as(
                        type_=BashResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def edit(
        self,
        instance_id: str,
        *,
        command: Command,
        path: str,
        file_text: typing.Optional[str] = OMIT,
        view_range: typing.Optional[typing.Sequence[int]] = OMIT,
        old_str: typing.Optional[str] = OMIT,
        new_str: typing.Optional[str] = OMIT,
        insert_line: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditResponse:
        """
        Parameters
        ----------
        instance_id : str

        command : Command

        path : str

        file_text : typing.Optional[str]

        view_range : typing.Optional[typing.Sequence[int]]

        old_str : typing.Optional[str]

        new_str : typing.Optional[str]

        insert_line : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.edit(
            instance_id="instance_id",
            command="view",
            path="path",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/edit",
            method="POST",
            json={
                "command": command,
                "path": path,
                "file_text": file_text,
                "view_range": view_range,
                "old_str": old_str,
                "new_str": new_str,
                "insert_line": insert_line,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EditResponse,
                    parse_obj_as(
                        type_=EditResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def file(
        self,
        instance_id: str,
        *,
        command: str,
        path: typing.Optional[str] = OMIT,
        content: typing.Optional[str] = OMIT,
        mode: typing.Optional[str] = OMIT,
        encoding: typing.Optional[str] = OMIT,
        view_range: typing.Optional[typing.Sequence[int]] = OMIT,
        recursive: typing.Optional[bool] = OMIT,
        src: typing.Optional[str] = OMIT,
        dst: typing.Optional[str] = OMIT,
        old_str: typing.Optional[str] = OMIT,
        new_str: typing.Optional[str] = OMIT,
        line: typing.Optional[int] = OMIT,
        text: typing.Optional[str] = OMIT,
        lines: typing.Optional[typing.Sequence[int]] = OMIT,
        all_occurrences: typing.Optional[bool] = OMIT,
        pattern: typing.Optional[str] = OMIT,
        case_sensitive: typing.Optional[bool] = OMIT,
        line_numbers: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileResponse:
        """
        Parameters
        ----------
        instance_id : str

        command : str

        path : typing.Optional[str]

        content : typing.Optional[str]

        mode : typing.Optional[str]

        encoding : typing.Optional[str]

        view_range : typing.Optional[typing.Sequence[int]]

        recursive : typing.Optional[bool]

        src : typing.Optional[str]

        dst : typing.Optional[str]

        old_str : typing.Optional[str]

        new_str : typing.Optional[str]

        line : typing.Optional[int]

        text : typing.Optional[str]

        lines : typing.Optional[typing.Sequence[int]]

        all_occurrences : typing.Optional[bool]

        pattern : typing.Optional[str]

        case_sensitive : typing.Optional[bool]

        line_numbers : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.file(
            instance_id="instance_id",
            command="command",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/file",
            method="POST",
            json={
                "command": command,
                "path": path,
                "content": content,
                "mode": mode,
                "encoding": encoding,
                "view_range": view_range,
                "recursive": recursive,
                "src": src,
                "dst": dst,
                "old_str": old_str,
                "new_str": new_str,
                "line": line,
                "text": text,
                "lines": lines,
                "all_occurrences": all_occurrences,
                "pattern": pattern,
                "case_sensitive": case_sensitive,
                "line_numbers": line_numbers,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    FileResponse,
                    parse_obj_as(
                        type_=FileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def stop(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StopInstanceResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.stop(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/stop",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StopInstanceResponse,
                    parse_obj_as(
                        type_=StopInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def pause(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StopInstanceResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.pause(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/pause",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StopInstanceResponse,
                    parse_obj_as(
                        type_=StopInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def resume(
        self,
        instance_id: str,
        *,
        timeout_hours: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        timeout_hours : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInstanceResponse
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.instance.resume(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/resume",
            method="POST",
            params={
                "timeout_hours": timeout_hours,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInstanceResponse,
                    parse_obj_as(
                        type_=GetInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncInstanceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def screenshot(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InstanceScreenshotResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InstanceScreenshotResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.screenshot(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/screenshot",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    InstanceScreenshotResponse,
                    parse_obj_as(
                        type_=InstanceScreenshotResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_stream_url(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InstanceGetStreamUrlResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InstanceGetStreamUrlResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.get_stream_url(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/stream_url",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    InstanceGetStreamUrlResponse,
                    parse_obj_as(
                        type_=InstanceGetStreamUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def computer(
        self, instance_id: str, *, request: Request, request_options: typing.Optional[RequestOptions] = None
    ) -> ComputerResponse:
        """
        Parameters
        ----------
        instance_id : str

        request : Request

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComputerResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara
        from scrapybara.instance import Request_MoveMouse

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.computer(
                instance_id="instance_id",
                request=Request_MoveMouse(
                    coordinates=[1],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/computer",
            method="POST",
            json=convert_and_respect_annotation_metadata(object_=request, annotation=Request, direction="write"),
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ComputerResponse,
                    parse_obj_as(
                        type_=ComputerResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def bash(
        self,
        instance_id: str,
        *,
        command: typing.Optional[str] = OMIT,
        restart: typing.Optional[bool] = OMIT,
        get_background_processes: typing.Optional[bool] = OMIT,
        kill_pid: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BashResponse:
        """
        Parameters
        ----------
        instance_id : str

        command : typing.Optional[str]

        restart : typing.Optional[bool]

        get_background_processes : typing.Optional[bool]

        kill_pid : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BashResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.bash(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/bash",
            method="POST",
            json={
                "command": command,
                "restart": restart,
                "get_background_processes": get_background_processes,
                "kill_pid": kill_pid,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    BashResponse,
                    parse_obj_as(
                        type_=BashResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def edit(
        self,
        instance_id: str,
        *,
        command: Command,
        path: str,
        file_text: typing.Optional[str] = OMIT,
        view_range: typing.Optional[typing.Sequence[int]] = OMIT,
        old_str: typing.Optional[str] = OMIT,
        new_str: typing.Optional[str] = OMIT,
        insert_line: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditResponse:
        """
        Parameters
        ----------
        instance_id : str

        command : Command

        path : str

        file_text : typing.Optional[str]

        view_range : typing.Optional[typing.Sequence[int]]

        old_str : typing.Optional[str]

        new_str : typing.Optional[str]

        insert_line : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.edit(
                instance_id="instance_id",
                command="view",
                path="path",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/edit",
            method="POST",
            json={
                "command": command,
                "path": path,
                "file_text": file_text,
                "view_range": view_range,
                "old_str": old_str,
                "new_str": new_str,
                "insert_line": insert_line,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    EditResponse,
                    parse_obj_as(
                        type_=EditResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def file(
        self,
        instance_id: str,
        *,
        command: str,
        path: typing.Optional[str] = OMIT,
        content: typing.Optional[str] = OMIT,
        mode: typing.Optional[str] = OMIT,
        encoding: typing.Optional[str] = OMIT,
        view_range: typing.Optional[typing.Sequence[int]] = OMIT,
        recursive: typing.Optional[bool] = OMIT,
        src: typing.Optional[str] = OMIT,
        dst: typing.Optional[str] = OMIT,
        old_str: typing.Optional[str] = OMIT,
        new_str: typing.Optional[str] = OMIT,
        line: typing.Optional[int] = OMIT,
        text: typing.Optional[str] = OMIT,
        lines: typing.Optional[typing.Sequence[int]] = OMIT,
        all_occurrences: typing.Optional[bool] = OMIT,
        pattern: typing.Optional[str] = OMIT,
        case_sensitive: typing.Optional[bool] = OMIT,
        line_numbers: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileResponse:
        """
        Parameters
        ----------
        instance_id : str

        command : str

        path : typing.Optional[str]

        content : typing.Optional[str]

        mode : typing.Optional[str]

        encoding : typing.Optional[str]

        view_range : typing.Optional[typing.Sequence[int]]

        recursive : typing.Optional[bool]

        src : typing.Optional[str]

        dst : typing.Optional[str]

        old_str : typing.Optional[str]

        new_str : typing.Optional[str]

        line : typing.Optional[int]

        text : typing.Optional[str]

        lines : typing.Optional[typing.Sequence[int]]

        all_occurrences : typing.Optional[bool]

        pattern : typing.Optional[str]

        case_sensitive : typing.Optional[bool]

        line_numbers : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.file(
                instance_id="instance_id",
                command="command",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/file",
            method="POST",
            json={
                "command": command,
                "path": path,
                "content": content,
                "mode": mode,
                "encoding": encoding,
                "view_range": view_range,
                "recursive": recursive,
                "src": src,
                "dst": dst,
                "old_str": old_str,
                "new_str": new_str,
                "line": line,
                "text": text,
                "lines": lines,
                "all_occurrences": all_occurrences,
                "pattern": pattern,
                "case_sensitive": case_sensitive,
                "line_numbers": line_numbers,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    FileResponse,
                    parse_obj_as(
                        type_=FileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def stop(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StopInstanceResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.stop(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/stop",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StopInstanceResponse,
                    parse_obj_as(
                        type_=StopInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def pause(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> StopInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StopInstanceResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.pause(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/pause",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    StopInstanceResponse,
                    parse_obj_as(
                        type_=StopInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def resume(
        self,
        instance_id: str,
        *,
        timeout_hours: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetInstanceResponse:
        """
        Parameters
        ----------
        instance_id : str

        timeout_hours : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInstanceResponse
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.instance.resume(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/resume",
            method="POST",
            params={
                "timeout_hours": timeout_hours,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetInstanceResponse,
                    parse_obj_as(
                        type_=GetInstanceResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



from typing import Literal
from .auth_state_response import AuthStateResponse
from .bash_response import BashResponse
from .browser_authenticate_response import BrowserAuthenticateResponse
from .browser_get_cdp_url_response import BrowserGetCdpUrlResponse
from .browser_get_current_url_response import BrowserGetCurrentUrlResponse
from .button import Button
from .cell_type import CellType
from .click_mouse_action import ClickMouseAction
from .click_mouse_action_click_type import ClickMouseActionClickType
from .computer_response import ComputerResponse
from .deployment_config_instance_type import DeploymentConfigInstanceType
from .drag_mouse_action import DragMouseAction
from .edit_response import EditResponse
from .env_get_response import EnvGetResponse
from .env_response import EnvResponse
from .execute_cell_request import ExecuteCellRequest
from .file_response import FileResponse
from .get_cursor_position_action import GetCursorPositionAction
from .get_instance_response import GetInstanceResponse
from .get_instance_response_instance_type import GetInstanceResponseInstanceType
from .http_validation_error import HttpValidationError
from .instance_get_stream_url_response import InstanceGetStreamUrlResponse
from .instance_screenshot_response import InstanceScreenshotResponse
from .kernel_info import KernelInfo
from .modify_browser_auth_response import ModifyBrowserAuthResponse
from .move_mouse_action import MoveMouseAction
from .notebook import Notebook
from .notebook_cell import NotebookCell
from .press_key_action import PressKeyAction
from .save_browser_auth_response import SaveBrowserAuthResponse
from .scroll_action import ScrollAction
from .start_browser_response import StartBrowserResponse
from .status import Status
from .stop_browser_response import StopBrowserResponse
from .stop_instance_response import StopInstanceResponse
from .take_screenshot_action import TakeScreenshotAction
from .type_text_action import TypeTextAction
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .wait_action import WaitAction
from .act import (
    TextPart,
    ImagePart,
    ToolCallPart,
    ToolResultPart,
    UserMessage,
    AssistantMessage,
    ToolMessage,
    Message,
    Model,
    SingleActRequest,
    TokenUsage,
    SingleActResponse,
    Step,
    ActResponse,
)
from .tool import Tool, ApiTool

Action = Literal[
    "move_mouse",
    "click_mouse",
    "drag_mouse",
    "scroll",
    "press_key",
    "type_text",
    "wait",
    "take_screenshot",
    "get_cursor_position",
]

__all__ = [
    "ActResponse",
    "Action",
    "ApiTool",
    "AssistantMessage",
    "AuthStateResponse",
    "BashResponse",
    "BrowserAuthenticateResponse",
    "BrowserGetCdpUrlResponse",
    "BrowserGetCurrentUrlResponse",
    "Button",
    "CellType",
    "ClickMouseAction",
    "ClickMouseActionClickType",
    "ComputerResponse",
    "DeploymentConfigInstanceType",
    "DragMouseAction",
    "EditResponse",
    "EnvGetResponse",
    "EnvResponse",
    "ExecuteCellRequest",
    "FileResponse",
    "GetCursorPositionAction",
    "GetInstanceResponse",
    "GetInstanceResponseInstanceType",
    "HttpValidationError",
    "ImagePart",
    "InstanceGetStreamUrlResponse",
    "InstanceScreenshotResponse",
    "KernelInfo",
    "Message",
    "Model",
    "ModifyBrowserAuthResponse",
    "MoveMouseAction",
    "Notebook",
    "NotebookCell",
    "PressKeyAction",
    "SaveBrowserAuthResponse",
    "ScrollAction",
    "SingleActRequest",
    "SingleActResponse",
    "StartBrowserResponse",
    "Status",
    "Step",
    "StopBrowserResponse",
    "StopInstanceResponse",
    "TakeScreenshotAction",
    "TextPart",
    "Tool",
    "ToolCallPart",
    "ToolMessage",
    "ToolResultPart",
    "TokenUsage",
    "TypeTextAction",
    "UserMessage",
    "ValidationError",
    "ValidationErrorLocItem",
    "WaitAction",
]



# This file was auto-generated by Fern from our API Definition.

import typing

Status = typing.Union[typing.Literal["deploying", "running", "paused", "terminated", "error", "warm_pool"], typing.Any]



from typing import Any, Dict, Optional, Type
from pydantic import BaseModel


class Tool(BaseModel):
    name: str
    description: Optional[str] = None
    parameters: Optional[Type[BaseModel]] = None

    def __call__(self, **kwargs: Any) -> Any:
        """Execute the tool with the given arguments.

        The kwargs type will be inferred from the parameters field's type hints.
        """
        raise NotImplementedError("Tool.__call__ must be implemented by subclasses")


class ApiTool(BaseModel):
    """A tool that can be serialized to JSON for API calls."""

    name: str
    description: Optional[str] = None
    parameters: Optional[Dict[str, Any]] = None

    @classmethod
    def from_tool(cls, tool: Tool) -> "ApiTool":
        """Convert a Tool to an ApiTool for API serialization."""
        return cls(
            name=tool.name,
            description=tool.description,
            parameters=tool.parameters.model_json_schema() if tool.parameters else None,
        )



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.kernel_info import KernelInfo
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.notebook import Notebook
from ..types.cell_type import CellType
from ..types.notebook_cell import NotebookCell
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NotebookClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_kernels(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[KernelInfo]:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[KernelInfo]
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.list_kernels(
            instance_id="instance_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/kernels",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[KernelInfo],
                    parse_obj_as(
                        type_=typing.List[KernelInfo],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        instance_id: str,
        *,
        name: str,
        kernel_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Notebook:
        """
        Parameters
        ----------
        instance_id : str

        name : str

        kernel_name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Notebook
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.create(
            instance_id="instance_id",
            name="name",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/create",
            method="POST",
            json={
                "name": name,
                "kernel_name": kernel_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Notebook,
                    parse_obj_as(
                        type_=Notebook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, instance_id: str, notebook_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Notebook:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Notebook
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.get(
            instance_id="instance_id",
            notebook_id="notebook_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Notebook,
                    parse_obj_as(
                        type_=Notebook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, instance_id: str, notebook_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.delete(
            instance_id="instance_id",
            notebook_id="notebook_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/delete",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def add_cell(
        self,
        instance_id: str,
        notebook_id: str,
        *,
        type: CellType,
        content: str,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NotebookCell:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        type : CellType

        content : str

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotebookCell
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.add_cell(
            instance_id="instance_id",
            notebook_id="notebook_id",
            type="code",
            content="content",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/cell",
            method="POST",
            json={
                "type": type,
                "content": content,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    NotebookCell,
                    parse_obj_as(
                        type_=NotebookCell,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def execute_cell(
        self,
        instance_id: str,
        notebook_id: str,
        cell_id: str,
        *,
        timeout: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NotebookCell:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        cell_id : str

        timeout : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotebookCell
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.execute_cell(
            instance_id="instance_id",
            notebook_id="notebook_id",
            cell_id="cell_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/cell/{jsonable_encoder(cell_id)}/execute",
            method="POST",
            json={
                "timeout": timeout,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    NotebookCell,
                    parse_obj_as(
                        type_=NotebookCell,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def execute(
        self,
        instance_id: str,
        notebook_id: str,
        *,
        timeout: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[NotebookCell]:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        timeout : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[NotebookCell]
            Successful Response

        Examples
        --------
        from scrapybara import Scrapybara

        client = Scrapybara(
            api_key="YOUR_API_KEY",
        )
        client.notebook.execute(
            instance_id="instance_id",
            notebook_id="notebook_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/execute",
            method="POST",
            json={
                "timeout": timeout,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[NotebookCell],
                    parse_obj_as(
                        type_=typing.List[NotebookCell],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncNotebookClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_kernels(
        self, instance_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[KernelInfo]:
        """
        Parameters
        ----------
        instance_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[KernelInfo]
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.list_kernels(
                instance_id="instance_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/kernels",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[KernelInfo],
                    parse_obj_as(
                        type_=typing.List[KernelInfo],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        instance_id: str,
        *,
        name: str,
        kernel_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Notebook:
        """
        Parameters
        ----------
        instance_id : str

        name : str

        kernel_name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Notebook
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.create(
                instance_id="instance_id",
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/create",
            method="POST",
            json={
                "name": name,
                "kernel_name": kernel_name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Notebook,
                    parse_obj_as(
                        type_=Notebook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, instance_id: str, notebook_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Notebook:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Notebook
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.get(
                instance_id="instance_id",
                notebook_id="notebook_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Notebook,
                    parse_obj_as(
                        type_=Notebook,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, instance_id: str, notebook_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Optional[typing.Any]]:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Optional[typing.Any]]
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.delete(
                instance_id="instance_id",
                notebook_id="notebook_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/delete",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.Optional[typing.Any]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Optional[typing.Any]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def add_cell(
        self,
        instance_id: str,
        notebook_id: str,
        *,
        type: CellType,
        content: str,
        metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NotebookCell:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        type : CellType

        content : str

        metadata : typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotebookCell
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.add_cell(
                instance_id="instance_id",
                notebook_id="notebook_id",
                type="code",
                content="content",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/cell",
            method="POST",
            json={
                "type": type,
                "content": content,
                "metadata": metadata,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    NotebookCell,
                    parse_obj_as(
                        type_=NotebookCell,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def execute_cell(
        self,
        instance_id: str,
        notebook_id: str,
        cell_id: str,
        *,
        timeout: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> NotebookCell:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        cell_id : str

        timeout : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        NotebookCell
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.execute_cell(
                instance_id="instance_id",
                notebook_id="notebook_id",
                cell_id="cell_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/cell/{jsonable_encoder(cell_id)}/execute",
            method="POST",
            json={
                "timeout": timeout,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    NotebookCell,
                    parse_obj_as(
                        type_=NotebookCell,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def execute(
        self,
        instance_id: str,
        notebook_id: str,
        *,
        timeout: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[NotebookCell]:
        """
        Parameters
        ----------
        instance_id : str

        notebook_id : str

        timeout : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[NotebookCell]
            Successful Response

        Examples
        --------
        import asyncio

        from scrapybara import AsyncScrapybara

        client = AsyncScrapybara(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.notebook.execute(
                instance_id="instance_id",
                notebook_id="notebook_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/instance/{jsonable_encoder(instance_id)}/notebook/{jsonable_encoder(notebook_id)}/execute",
            method="POST",
            json={
                "timeout": timeout,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[NotebookCell],
                    parse_obj_as(
                        type_=typing.List[NotebookCell],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from ...types.button import Button
from ...types.click_mouse_action_click_type import ClickMouseActionClickType


class Request_MoveMouse(UniversalBaseModel):
    action: typing.Literal["move_mouse"] = "move_mouse"
    coordinates: typing.List[int]
    hold_keys: typing.Optional[typing.List[str]] = None
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_ClickMouse(UniversalBaseModel):
    action: typing.Literal["click_mouse"] = "click_mouse"
    button: Button
    click_type: typing.Optional[ClickMouseActionClickType] = None
    coordinates: typing.Optional[typing.List[int]] = None
    num_clicks: typing.Optional[int] = None
    hold_keys: typing.Optional[typing.List[str]] = None
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_DragMouse(UniversalBaseModel):
    action: typing.Literal["drag_mouse"] = "drag_mouse"
    path: typing.List[typing.List[int]]
    hold_keys: typing.Optional[typing.List[str]] = None
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_Scroll(UniversalBaseModel):
    action: typing.Literal["scroll"] = "scroll"
    coordinates: typing.Optional[typing.List[int]] = None
    delta_x: typing.Optional[float] = None
    delta_y: typing.Optional[float] = None
    hold_keys: typing.Optional[typing.List[str]] = None
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_PressKey(UniversalBaseModel):
    action: typing.Literal["press_key"] = "press_key"
    keys: typing.List[str]
    duration: typing.Optional[float] = None
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_TypeText(UniversalBaseModel):
    action: typing.Literal["type_text"] = "type_text"
    text: str
    hold_keys: typing.Optional[typing.List[str]] = None
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_Wait(UniversalBaseModel):
    action: typing.Literal["wait"] = "wait"
    duration: float
    screenshot: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_TakeScreenshot(UniversalBaseModel):
    action: typing.Literal["take_screenshot"] = "take_screenshot"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Request_GetCursorPosition(UniversalBaseModel):
    action: typing.Literal["get_cursor_position"] = "get_cursor_position"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


Request = typing.Union[
    Request_MoveMouse,
    Request_ClickMouse,
    Request_DragMouse,
    Request_Scroll,
    Request_PressKey,
    Request_TypeText,
    Request_Wait,
    Request_TakeScreenshot,
    Request_GetCursorPosition,
]



from typing import Literal, Optional

from pydantic import Field

from ..types.act import Model
from datetime import datetime


class Anthropic(Model):
    """Model adapter for Anthropic.

    Supported models:
    - claude-3-7-sonnet-20250219 (1x agent credit if no api_key)
    - claude-3-7-sonnet-20250219-thinking (1x agent credit if no api_key)
    - claude-3-5-sonnet-20241022 (1x agent credit if no api_key)

    Args:
        name: Anthropic model name, defaults to "claude-3-7-sonnet-20250219"
        api_key: Optional Anthropic API key

    Returns:
        A Model configuration object
    """

    provider: Literal["anthropic"] = Field(default="anthropic", frozen=True)

    def __init__(
        self,
        name: Optional[str] = "claude-3-7-sonnet-20250219",
        api_key: Optional[str] = None,
    ) -> None:
        super().__init__(provider="anthropic", name=name, api_key=api_key)


UBUNTU_SYSTEM_PROMPT = f"""<SYSTEM_CAPABILITY>
* You have access to an Ubuntu VM with internet connectivity
* You can install Ubuntu applications using the bash tool (use curl over wget)
* To run GUI applications with the bash tool, use a subshell, e.g. "(DISPLAY=:1 xterm &)", make sure to include the parantheses
* GUI apps will appear but may take time to load - confirm with an extra screenshot
* Chromium is the default browser
* Start Chromium via the bash tool "(DISPLAY=:1 chromium &)", but interact with it visually via the computer tool
* If you need to read a HTML file:
  - Open with the address bar in Chromium
* For commands with large text output:
  - Redirect to a temp file
  - Use str_replace_editor or grep with context (-B and -A flags) to view output
* When viewing pages:
  - Zoom out to see full content, or
  - Scroll to ensure you see everything
* When interacting with a field, always clear the field first using "ctrl+A" and "delete"
  - Take an extra screenshot after clicking "enter" to confirm the field is properly submitted and move the mouse to the next field
* Computer function calls take time, string together calls when possible
* You are allowed to take actions on behalf of the user on sites that are authenticated
* If the user asks you to access a site, assume that the user has already authenticated
* To login additional sites, ask the user to use Auth Contexts or the Interactive Desktop
* If first screenshot shows black screen:
  - Click mouse in screen center
  - Take another screenshot
* Today's date is {datetime.today().strftime('%A, %B %d, %Y')}
</SYSTEM_CAPABILITY>

<IMPORTANT>
* If given a complex task, break down into smaller steps and ask the user for details only if necessary
* Read through web pages thoroughly by scrolling down till you have gathered enough info
* Be concise!
</IMPORTANT>"""
"""Recommended Anthropic system prompt for Ubuntu instances"""


BROWSER_SYSTEM_PROMPT = f"""<SYSTEM_CAPABILITY>
* You have access to a Chromium VM with internet connectivity
* Chromium should already be open and running
* You can interact with web pages using the computer tool
* When viewing pages:
  - Zoom out to see full content, or
  - Scroll to ensure you see everything
* When interacting with a field, always clear the field first using "ctrl+A" and "delete"
  - Take an extra screenshot after clicking "enter" to confirm the field is properly submitted and move the mouse to the next field
* Computer function calls take time, string together calls when possible
* You are allowed to take actions on behalf of the user on sites that are authenticated
* If the user asks you to access a site, assume that the user has already authenticated
* To login additional sites, ask the user to use Auth Contexts
* If first screenshot shows black screen:
  - Click mouse in screen center
  - Take another screenshot
* Today's date is {datetime.today().strftime('%A, %B %d, %Y')}
</SYSTEM_CAPABILITY>

<IMPORTANT>
* If given a complex task, break down into smaller steps and ask the user for details only if necessary
* Read through web pages thoroughly by scrolling down till you have gathered enough info
* Be concise!
</IMPORTANT>"""
"""Recommended Anthropic system prompt for Browser instances"""


WINDOWS_SYSTEM_PROMPT = f"""<SYSTEM_CAPABILITY>
* You wave access to a Windows VM with internet connectivity
* You can interact with the Windows desktop using the computer tool
* GUI apps will appear but may take time to load - confirm with an extra screenshot
* Edge is the default browser
* When viewing pages:
  - Zoom out to see full content, or
  - Scroll to ensure you see everything
* When interacting with a field, always clear the field first using "ctrl+A" and "delete"
  - Take an extra screenshot after clicking "enter" to confirm the field is properly submitted and move the mouse to the next field
* Computer function calls take time, string together calls when possible
* You are allowed to take actions on behalf of the user on sites that are authenticated
* If the user asks you to access a site, assume that the user has already authenticated
* To login additional sites, ask the user to use Auth Contexts or the Interactive Desktop
* If first screenshot shows black screen:
  - Click mouse in screen center
  - Take another screenshot
* Today's date is {datetime.today().strftime('%A, %B %d, %Y')}
</SYSTEM_CAPABILITY>

<IMPORTANT>
* If given a complex task, break down into smaller steps and ask the user for details only if necessary
* Read through web pages thoroughly by scrolling down till you have gathered enough info
* Be concise!
</IMPORTANT>"""
"""Recommended Anthropic system prompt for Windows instances"""

STRUCTURED_OUTPUT_SECTION = """
* When you have completed your task and are ready to provide the final result to the user, use the 'structured_output' tool
* This tool allows you to output structured data according to the provided schema
* Ensure that your output matches the expected schema by providing the correct fields and data types
* The output from this tool will be passed directly back to the user as the final result
* Do not present the final result in plain text; always use the 'structured_output' tool for the final output
"""
"""Section to add to system prompt when structured output is being used"""